@@ -19,9 +19,14 @@ Alien::Gnuplot - Find and verify functionality of the gnuplot executable.
 =head1 DESCRIPTION
 
 Alien::Gnuplot verifies existence and sanity of the gnuplot external
-application.  It doesn't have any methods - actually using gnuplot is
-up to you.  Using Alien::Gnuplot checks for existence of the
-executable, and sets several global variables:
+application.  It only declares one access method,
+C<Alien::Gnuplot::load_gnuplot>, which does the actual work and is
+called automatically at load time.  Alien::Gnuplot doesn't have any
+actual plotting methods - making use of gnuplot, once it is found and
+verified, is up to you or your client module.
+
+Using Alien::Gnuplot checks for existence of the executable, and sets
+several global variables:
 
 =over 3
 
@@ -35,6 +40,10 @@ executable, and sets several global variables:
 
 =item * C<%Alien::Gnuplot::terms> gets a key for each supported terminal device; values are the 1-line description from gnuplot.
 
+=item * C<@Alien::Gnuplot::colors> gets a list of the names of all named colors recognized by this gnuplot.
+
+=item * C<%Alien::Gnuplot::colors> gets a key for each named color; values are the C<#RRGGBB> form of the color.
+
 =back
 
 You can point Alien::Gnuplot to a particular path for gnuplot, by
@@ -46,6 +55,11 @@ You can also verify that it has not completed successfully, by
 examining $Alien::Gnuplot::version, which is undefined in case of
 failure and contains the gnuplot version string on success.
 
+If you think the global state of the gnuplot executable may have
+changed, you can either reload the module or explicitly call
+C<Alien::Gnuplot::load_gnuplot()> to force a fresh inspection of
+the executable.
+
 =head1 INSTALLATION STRATEGY
 
 When you install Alien::Gnuplot, it checks that gnuplot itself is
@@ -90,7 +104,7 @@ use POSIX ":sys_wait_h";
 # overload the system VERSION to compare a required version against gnuplot itself, rather
 # than against the module version.
 
-our $VERSION = '1.000';
+our $VERSION = '1.001';
 
 # On install, try to make sure at least this version is present.
 our $GNUPLOT_RECOMMENDED_VERSION = '4.6';  
@@ -100,6 +114,8 @@ our $version;     # Holds the found version number
 our $pl;          # Holds the found patchlevel
 our @terms;
 our %terms;
+our @colors;
+our %colors;
 
 sub VERSION {
     my $module =shift;
@@ -116,119 +132,138 @@ getting it yourself from L<http://www.gnuplot.info>.
 }
 
 
+sub load_gnuplot {
 ##############################
 # Search the path for the executable
 #
-my $exec_path;
-if($ENV{'GNUPLOT_BINARY'}) {
-   $exec_path = $ENV{'GNUPLOT_BINARY'};
-} else {
-    my $exec_str = "gnuplot";
-    if( defined($ENV{'PATH'}) ) {
-	# POSIX path present...
-	my @path = split (/\:/,$ENV{'PATH'});
-	for my $dir(@path) {
-	    $exec_path = "$dir/$exec_str";
-	    last if( -x $exec_path );
-	}
+    my $exec_path;
+    if($ENV{'GNUPLOT_BINARY'}) {
+	$exec_path = $ENV{'GNUPLOT_BINARY'};
     } else {
-	die "Alien::Gnuplot: No POSIX-style path found, and no GNUPLOT_BINARY environment\nvariable found either\n\n";
+	my $exec_str = "gnuplot";
+	if( defined($ENV{'PATH'}) ) {
+	    # POSIX path present...
+	    my @path = split (/\:/,$ENV{'PATH'});
+	    for my $dir(@path) {
+		$exec_path = "$dir/$exec_str";
+		last if( -x $exec_path );
+	    }
+	} else {
+	    die "Alien::Gnuplot: No POSIX-style path found, and no GNUPLOT_BINARY environment\nvariable found either\n\n";
+	}
     }
-}
-
-unless(-x $exec_path) { 
-    die q{
+    
+    unless(-x $exec_path) { 
+	die q{
 Alien::Gnuplot: no executable gnuplot found!  If you have gnuplot,
 you can put its exact location in your GNUPLOT_BINARY environment 
 variable or make sure your PATH contains it.  If you do not have
 gnuplot, you can reinstall Alien::Gnuplot to get it, or get
 it yourself from L<http:/www.gnuplot.info>.
 };
-}
-
-
+    }
+    
+    
 ##############################
 # Execute the executable to make sure it's really gnuplot, and parse
 # out its reported version.  This is complicated by gnuplot's shenanigans
 # with STDOUT and STDERR, so we fork and redirect everything to a file.
 # The parent process gives the daughter 2 seconds to report progress, then
 # kills it dead.
-my($pid);
-my ($undef, $file) = tempfile('gnuplot_test_XXXX');
-
-$pid = fork();
-if(defined($pid) and !$pid) {
-    # daughter
-    open STDOUT, ">$file";
-    open STDERR, ">&STDOUT";
-    open FOO, "|$exec_path";
-    print FOO "show version\nset terminal\n\n\n\n\n\n\n\n\n\n";
-    close FOO;
-    exit(0);
-}
-elsif($pid>0) {
-    # Poll for 2 seconds, cheesily.
-    for (1..20) {
-	if(waitpid($pid,WNOHANG)) {
-	    $pid=0;
-	    last;
+    my($pid);
+    my ($undef, $file) = tempfile('gnuplot_test_XXXX');
+    
+    $pid = fork();
+    if(defined($pid) and !$pid) {
+	# daughter
+	open STDOUT, ">$file";
+	open STDERR, ">&STDOUT";
+	open FOO, "|$exec_path";
+	print FOO "show version\nset terminal\n\n\n\n\n\n\n\n\n\nprint \"CcColors\"\nshow colornames\n\n\n\n\n\n\n\nprint \"FfFinished\"n";
+	close FOO;
+	exit(0);
+    }
+    elsif($pid>0) {
+	# Poll for 2 seconds, cheesily.
+	for (1..20) {
+	    if(waitpid($pid,WNOHANG)) {
+		$pid=0;
+		last;
+	    }
+	    usleep(1e5);
 	}
-	usleep(1e5);
+	
+	if($pid) {
+	    kill 9,$pid;   # zap
+	    waitpid($pid,0); # reap
+	}
+    } else {
+	die "Couldn't fork!";
     }
-
-    if($pid) {
-	kill 9,$pid;   # zap
-	waitpid($pid,0); # reap
-    }
-} else {
-    die "Couldn't fork!";
-}
-
+    
 ##############################
 # Read what gnuplot had to say, and clean up our mess...
-open FOO, "<$file";
-my @lines = <FOO>;
-unlink $file;
-
-
+    open FOO, "<$file";
+    my @lines = <FOO>;
+    unlink $file;
+    
+    
 ##############################
 # Whew.  Now parse out the 'GNUPLOT' and version number...
-my $lines = join("", map { chomp $_; $_} @lines);
-$lines =~ s/\s+G N U P L O T\s*//  or  die qq{
+    my $lines = join("", map { chomp $_; $_} @lines);
+    $lines =~ s/\s+G N U P L O T\s*//  or  die qq{
 Alien::Gnuplot: the executable file $exec_path appears not to be gnuplot!  You can 
 remove it or set your GNUPLOT_BINARY variable to an actual gnuplot.
 
 };
-
-$lines =~ m/Version (\d+\.\d+) (patchlevel (\d+))?/ or die qq{
+    
+    $lines =~ m/Version (\d+\.\d+) (patchlevel (\d+))?/ or die qq{
 Alien::Gnuplot: the executable file $exec_path claims to be gnuplot, but 
 I could not parse a verion number from its output.  Sorry, I give up.
 
 };
-
-$version = $1;
-$pl = $3;
-$executable = $exec_path;
-
-
+    
+    $version = $1;
+    $pl = $3;
+    $executable = $exec_path;
+    
+    
 ##############################
 # Parse out available terminals and put them into the 
 # global list and hash.
-@terms = ();
-%terms = ();
-my $reading_terms = 0;
-for my $line(@lines) {
-    if(!$reading_terms) {
-	if($line =~ m/^Available terminal types\:/) {
-	    $reading_terms = 1;
+    @terms = ();
+    %terms = ();
+    my $reading_terms = 0;
+    for my $line(@lines) {
+	last if($line =~ m/CcColors/);
+	if(!$reading_terms) {
+	    if($line =~ m/^Available terminal types\:/) {
+		$reading_terms = 1;
+	    }
+	} else {
+	    next if($line =~ m/^Press return for more/);
+	    $line =~ m/^\s*(\w+)\s(.*[^\s])\s*$/ || last;
+	    push(@terms, $1);
+	    $terms{$1} = $2;
 	}
-    } else {
-	next if($line =~ m/^Press return for more/);
-	$line =~ m/^\s*(\w+)\s(.*[^\s])\s*$/ || last;
-	push(@terms, $1);
-	$terms{$1} = $2;
     }
+    
+##############################
+# Parse out available colors and put them into that global list and hash.
+    @colors = ();
+    %colors = ();
+    
+    for my $line(@lines) {
+	last if($line =~ m/FfFinished/);
+	next unless( $line =~ m/\s+([\w\-0-9]+)\s+(\#......)/);
+	$colors{$1} = $2;
+    }
+    @colors = sort keys %colors;
 }
 
+
+load_gnuplot();
+
+
 1;
 
